/*
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.0 of the PHP license,       |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | http://www.php.net/license/3_0.txt.                                  |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Authors: Ryusuke SEKIYAMA <rsky0711@gmail.com>                       |
   +----------------------------------------------------------------------+
*/

/* $ Id: $ */ 

#include "php_helloworld.h"

#if HAVE_HELLOWORLD

#include <ext/standard/php_smart_str.h>
/* {{{ helloworld_functions[] */
function_entry helloworld_functions[] = {
	PHP_FE(helloworld          , helloworld_arg_info)
	PHP_FE(helloworld_get      , helloworld_get_arg_info)
	PHP_FE(helloworld_set      , helloworld_set_arg_info)
	PHP_FE(helloworld_put      , helloworld_put_arg_info)
	PHP_FE(helloworld_repeat   , helloworld_repeat_arg_info)
	PHP_FE(helloworld_array    , helloworld_array_arg_info)
	PHP_FE(helloworld_in_array , helloworld_in_array_arg_info)
	PHP_FE(helloworld_key_exists, helloworld_key_exists_arg_info)
	PHP_FE(helloworld_say      , helloworld_say_arg_info)
	PHP_FE(helloworld_apply    , helloworld_apply_arg_info)
	{ NULL, NULL, NULL }
};
/* }}} */


/* {{{ helloworld_module_entry
 */
zend_module_entry helloworld_module_entry = {
	STANDARD_MODULE_HEADER,
	"helloworld",
	helloworld_functions,
	PHP_MINIT(helloworld),     /* Replace with NULL if there is nothing to do at php startup   */ 
	PHP_MSHUTDOWN(helloworld), /* Replace with NULL if there is nothing to do at php shutdown  */
	PHP_RINIT(helloworld),     /* Replace with NULL if there is nothing to do at request start */
	PHP_RSHUTDOWN(helloworld), /* Replace with NULL if there is nothing to do at request end   */
	PHP_MINFO(helloworld),
	"1.0.0", 
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_HELLOWORLD
ZEND_GET_MODULE(helloworld)
#endif


/* {{{ PHP_MINIT_FUNCTION */
PHP_MINIT_FUNCTION(helloworld)
{
	REGISTER_LONG_CONSTANT("HW_INT", 1, CONST_PERSISTENT | CONST_CS);
	REGISTER_DOUBLE_CONSTANT("HW_FLT", 1.0, CONST_PERSISTENT | CONST_CS);
	REGISTER_STRINGL_CONSTANT("HW_STR", "Hello, World!", 13, CONST_PERSISTENT | CONST_CS);

	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_MSHUTDOWN_FUNCTION */
PHP_MSHUTDOWN_FUNCTION(helloworld)
{

	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_RINIT_FUNCTION */
PHP_RINIT_FUNCTION(helloworld)
{
	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_RSHUTDOWN_FUNCTION */
PHP_RSHUTDOWN_FUNCTION(helloworld)
{
	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_MINFO_FUNCTION */
PHP_MINFO_FUNCTION(helloworld)
{
	php_info_print_box_start(0);
	php_printf("<p>Hello, World!</p>\n");
	php_printf("<p>Version 1.0.0stable (2007-09-01)</p>\n");
	php_printf("<p><b>Authors:</b></p>\n");
	php_printf("<p>Ryusuke SEKIYAMA &lt;rsky0711@gmail.com&gt; (lead)</p>\n");
	php_info_print_box_end();
	do {
		php_info_print_table_start();
		php_info_print_table_row(2, "Hello World Extension", "enabled");
		php_info_print_table_row(2, "Module Version", "1.0.0");
		php_info_print_table_end();
	} while (0);

}
/* }}} */


/* {{{ proto void helloworld()
  echo Hello, World! */
PHP_FUNCTION(helloworld)
{



	if (ZEND_NUM_ARGS()>0)  {
		WRONG_PARAM_COUNT;
	}


	do {
		php_printf("Hello, World!");
	} while (0);
}
/* }}} helloworld */


/* {{{ proto string helloworld_get()
  get Hello, World! */
PHP_FUNCTION(helloworld_get)
{



	if (ZEND_NUM_ARGS()>0)  {
		WRONG_PARAM_COUNT;
	}


	do {
		RETURN_STRING("Hello, World!", 1);
	} while (0);
}
/* }}} helloworld_get */


/* {{{ proto void helloworld_set(mixed var)
  set Hello, World! */
PHP_FUNCTION(helloworld_set)
{

	zval * var = NULL;



	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z/", &var) == FAILURE) {
		return;
	}

	do {
		zval_dtor(var);
		ZVAL_STRING(var, "Hello, World!", 1);
	} while (0);
}
/* }}} helloworld_set */


/* {{{ proto int helloworld_put(string url)
  write Hello, World! */
PHP_FUNCTION(helloworld_put)
{

	const char * url = NULL;
	int url_len = 0;



	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &url, &url_len) == FAILURE) {
		return;
	}

	do {
		php_stream *stream;
		size_t len;
		
		stream = php_stream_open_wrapper((char *)url, "wb", ENFORCE_SAFE_MODE | REPORT_ERRORS, NULL);
		if (stream == NULL) {
			RETURN_FALSE;
		}
		
		len = php_stream_write_string(stream, "Hello, World!");
		
		php_stream_close(stream);
		
		RETURN_LONG((long)len);
	} while (0);
}
/* }}} helloworld_put */


/* {{{ proto string helloworld_repeat(int times[, string separator])
  repeat Hello, World! */
PHP_FUNCTION(helloworld_repeat)
{

	long times = 0;
	const char * separator = NULL;
	int separator_len = 0;



	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l|s", &times, &separator, &separator_len) == FAILURE) {
		return;
	}

	do {
		if (times < 0L) {
			php_error(E_WARNING, "Argument 1 should not be a negative number.");
			RETURN_FALSE;
		} else {
			smart_str str = {0};
			long n = 0L;
		
			while (n++ < times) {
				smart_str_appends(&str, "Hello, World!");
				if (separator != NULL && separator_len > 0) {
					smart_str_appendl(&str, separator, separator_len);
				} else {
					smart_str_appendc(&str, ' ');
				}
			}
		
			if (separator != NULL && separator_len > 0) {
				RETVAL_STRINGL(str.c, str.len - separator_len, 1);
			} else {
				RETVAL_STRINGL(str.c, str.len - 1, 1);
			}
			smart_str_free(&str);
		}
	} while (0);
}
/* }}} helloworld_repeat */


/* {{{ proto array helloworld_array(int times)
  array(Hello, World!) */
PHP_FUNCTION(helloworld_array)
{

	long times = 0;



	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &times) == FAILURE) {
		return;
	}

	array_init(return_value);

	do {
		if (times < 0L) {
			zval_dtor(return_value);
			php_error(E_WARNING, "Argument 1 should not be a negative number.");
			RETURN_FALSE;
		} else {
			long n = 0L;
			while (n++ < times) {
				add_next_index_string(return_value, "Hello, World!", 1);
			}
		}
	} while (0);
}
/* }}} helloworld_array */


/* {{{ proto bool helloworld_in_array(array arr[, bool ignore_case])
  find Hello, World! (value) */
PHP_FUNCTION(helloworld_in_array)
{

	zval * arr = NULL;
	HashTable * arr_hash = NULL;
	zend_bool ignore_case = 0;



	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a/|b", &arr, &ignore_case) == FAILURE) {
		return;
	}
	arr_hash = HASH_OF(arr);



	do {
		zval **entry = NULL;
		
		zend_hash_internal_pointer_reset(arr_hash);
		
		while (zend_hash_get_current_data(arr_hash, (void **)&entry) == SUCCESS) {
			if (Z_TYPE_PP(entry) == IS_STRING) {
				if (ignore_case) {
					if (!strcasecmp("Hello, World!", Z_STRVAL_PP(entry))) {
						RETURN_TRUE;
					}
				} else {
					if (!strcmp("Hello, World!", Z_STRVAL_PP(entry))) {
						RETURN_TRUE;
					}
				}
			}
			zend_hash_move_forward(arr_hash);
		}
		
		RETURN_FALSE;
	} while (0);
}
/* }}} helloworld_in_array */


/* {{{ proto bool helloworld_key_exists(array arr)
  find Hello, World! (key) */
PHP_FUNCTION(helloworld_key_exists)
{

	zval * arr = NULL;
	HashTable * arr_hash = NULL;



	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a/", &arr) == FAILURE) {
		return;
	}
	arr_hash = HASH_OF(arr);



	do {
		RETURN_BOOL(zend_hash_exists(arr_hash, "Hello, World!", sizeof("Hello, World!")));
	} while (0);
}
/* }}} helloworld_key_exists */


/* {{{ proto bool helloworld_say()
  say Hello, World! */
PHP_FUNCTION(helloworld_say)
{



	if (ZEND_NUM_ARGS()>0)  {
		WRONG_PARAM_COUNT;
	}


	do {
		zval **argv[] = { NULL, NULL, NULL };
		zval *command, *output, *result_code, *retval = NULL;
		zval func;
		
		MAKE_STD_ZVAL(command);
		MAKE_STD_ZVAL(output);
		MAKE_STD_ZVAL(result_code);
		
		ZVAL_STRING(command, "say 'Hello, World!'", 1);
		ZVAL_NULL(output);
		ZVAL_NULL(result_code);
		ZVAL_STRING(&func, "exec", 0);
		
		argv[0] = &command;
		argv[1] = &output;
		argv[2] = &result_code;
		
		if (call_user_function_ex(EG(function_table), NULL, &func,
				&retval, 3, argv, 0, NULL TSRMLS_CC) == FAILURE) {
			RETVAL_FALSE;
		} else if (Z_TYPE_P(result_code) == IS_LONG && Z_LVAL_P(result_code) == 0L) {
			RETVAL_TRUE; 
		} else {
			RETVAL_FALSE;
		}
		
		if (retval != NULL) {
			zval_ptr_dtor(&retval);
		}
		zval_ptr_dtor(&command);
		zval_ptr_dtor(&output);
		zval_ptr_dtor(&result_code);
	} while (0);
}
/* }}} helloworld_say */


/* {{{ proto mixed helloworld_apply(callback func[, array extra_args])
  apply Hello, World! */
PHP_FUNCTION(helloworld_apply)
{

	zval * func = NULL;
	zval * extra_args = NULL;
	HashTable * extra_args_hash = NULL;



	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z|a/", &func, &extra_args) == FAILURE) {
		return;
	}
	if (!zend_is_callable(func, 0, NULL)) {
	  php_error(E_WARNING, "Invalid comparison function.");
	  return;    }
	if (extra_args != NULL) {
		extra_args_hash = HASH_OF(extra_args);
	}



	do {
		zval ***argv, ***argp;
		zval *arg1;
		int argc = 1;
		zval *retval = NULL;
		
		if (extra_args_hash != NULL) {
			argc += zend_hash_num_elements(extra_args_hash);
		}
		
		argv = (zval ***)emalloc(sizeof(zval **) * argc);
		argp = argv;
		
		MAKE_STD_ZVAL(arg1);
		ZVAL_STRING(arg1, "Hello, World!", 1);
		*argp++ = &arg1;
		
		if (extra_args_hash != NULL) {
			zval **entry;
			zval *arg;
		
			zend_hash_internal_pointer_reset(extra_args_hash);
		
			while (zend_hash_get_current_data(extra_args_hash, (void **)&entry) == SUCCESS) {
				MAKE_STD_ZVAL(arg);
				ZVAL_ZVAL(arg, *entry, 1, 0);
				*argp++ = &arg;
				zend_hash_move_forward(extra_args_hash);
			}
		}
		
		if (call_user_function_ex(EG(function_table), NULL, func,
				&retval, (zend_uint)argc, argv, 0, NULL TSRMLS_CC) == FAILURE) {
			RETVAL_FALSE;
		} else {
			RETVAL_ZVAL(retval, 1, 1);
		}
		
		while (argp > argv) {
			zval_ptr_dtor(*(--argp));
		}
		efree(argv);
	} while (0);
}
/* }}} helloworld_apply */

#endif /* HAVE_HELLOWORLD */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
